//
// FunkyFeedbackView.h     1.0
//
// Created by Wagner Truppel on 2009-10-08.
// Copyright 2009 Wagner Truppel (jrcapab@gmail.com). All rights reserved.
//
// Redistribution and use in binary and source forms, with or without
// modification, are permitted for any project, commercial or otherwise,
// provided that the following conditions are met:
//
// Redistributions in binary form must display the copyright notice in the
// About view, website, and/or documentation.
//
// Redistributions of source code must retain the copyright notice, this
// list of conditions, and the following disclaimer.
//
// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

// This is an example of subclassing WTFeedbackView to change the
// appearance of its various components.

// FunkyFeedbackView is an extension of a bezel-style flexible
// WTFeedbackView with some interface changes.

#import <QuartzCore/QuartzCore.h>

#import "FunkyFeedbackView.h"





// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
                                #pragma mark -
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //

@interface FunkyFeedbackView (Private)

- (UIView*) borderView;

- (UILabel*) feedbackLabel;

@end

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
                                #pragma mark -
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //

@implementation FunkyFeedbackView

// ========================================================================= //
                         #pragma mark factory methods
// ========================================================================= //

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (FunkyFeedbackView*) newFunkyViewForView: (UIView*) addToView
                       showingProgressView: (BOOL) showing
                                  withText: (NSString*) labelText;
{
    // Not autoreleased because the feedback view is essentially a singleton.

    return [[[self class] alloc] initWithKind: WTFeedbackViewKindFlexible
                                        style: WTFeedbackViewStyleBezel
                                      forView: addToView
                                     withText: labelText
                          showingProgressView: showing];
}

// ========================================================================= //
                       #pragma mark overridable methods
// ========================================================================= //

- (void) setupBackground;
{
    [super setupBackground];

    self.backgroundColor = [[UIColor blueColor]
        colorWithAlphaComponent: 0.5];
}

// ========================================================================= //
                        #pragma mark private methods
// ========================================================================= //

- (UIView*) borderView;
{
    if (_borderView == nil)
    {
        _borderView = [super borderView];

        _borderView.backgroundColor = [[UIColor whiteColor]
            colorWithAlphaComponent: 0.75];

        _borderView.layer.cornerRadius = 5.0;
    }

    return _borderView;
}

// ========================================================================= //

- (UILabel*) feedbackLabel;
{
    if (_feedbackLabel == nil)
    {
        _feedbackLabel = [super feedbackLabel];
        _feedbackLabel.backgroundColor = [UIColor clearColor];

        _feedbackLabel.font = [UIFont systemFontOfSize: 18.0];

        _feedbackLabel.textColor    = [UIColor blueColor];
        _feedbackLabel.shadowColor  = [UIColor blackColor];
        _feedbackLabel.shadowOffset = CGSizeMake(0.0, 1.0);
    }

    return _feedbackLabel;
}

// ========================================================================= //

@end

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
