//
// DONE
//
// RootViewController.m
// WTFeedbackViewDemo   1.0
//
// Created by Wagner Truppel on 2009-10-08.
// Copyright 2009 Wagner Truppel (jrcapab@gmail.com). All rights reserved.
//
// Redistribution and use in binary and source forms, with or without
// modification, are permitted for any project, commercial or otherwise,
// provided that the following conditions are met:
//
// Redistributions in binary form must display the copyright notice in the
// About view, website, and/or documentation.
//
// Redistributions of source code must retain the copyright notice, this
// list of conditions, and the following disclaimer.
//
// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THIS SOFTWARE.

#import "CustomLogging.h"
#import "RootViewController.h"
#import "DemoViewController.h"
#import "RootViewSection.h"
#import "RootViewCell.h"
#import "Enums.h"



// ========================================================================= //
                       #pragma mark private interface
// ========================================================================= //

@interface RootViewController ()

@property (readwrite, nonatomic, retain) RootViewSection* customSectionView;
@property (readwrite, nonatomic, retain) RootViewCell*    customCellView;

@property (readwrite, nonatomic, retain) NSMutableArray* cachedSectionViews;

@end

// ========================================================================= //

@interface RootViewController (Private)

- (RootViewCell*) tableView: (UITableView*) table_view
         loadCellForSection: (NSInteger) section
                        row: (NSInteger) row;

- (RootViewCell*) tableView: (UITableView*) table_view
                 loadCellID: (NSString*) cellID
                   nibNamed: (NSString*) nibName;

@end

// ========================================================================= //



@implementation RootViewController

// ========================================================================= //
                           #pragma mark properties
// ========================================================================= //

@synthesize customSectionView;
@synthesize customCellView;

@synthesize cachedSectionViews;

// ========================================================================= //
                            #pragma mark methods
// ========================================================================= //

- (BOOL) shouldAutorotateToInterfaceOrientation:
            (UIInterfaceOrientation) interfaceOrientation;
{
    BEGIN
    // WTLOG(@"RootViewController: -shouldAutorotateToInterfaceOrientation");
    RETURN YES;
    END
}

// ========================================================================= //

- (void) viewDidLoad;
{
    BEGIN
    // WTLOG(@"RootViewController: -viewDidLoad");

    [super viewDidLoad];

    // Create and then fill the section views cache with
    // NSNull references.
    if (cachedSectionViews == nil)
    {
        cachedSectionViews = [[NSMutableArray alloc]
            initWithCapacity: NUM_SECTIONS];

        for (NSInteger i = 0; i < NUM_SECTIONS; ++i)
        { [cachedSectionViews addObject: [NSNull null]]; }
    }

    END
}

// ========================================================================= //

- (void) viewDidUnload;
{
    BEGIN
    // WTLOG(@"RootViewController: -viewDidUnload");

    // Dispose of the section views cache.
    [cachedSectionViews removeAllObjects];
    self.cachedSectionViews = nil;

    [super viewDidUnload];

    END
}

// ========================================================================= //

- (void) dealloc;
{
    BEGIN
    // WTLOG(@"RootViewController: -shouldAutorotateToInterfaceOrientation");

    [customSectionView  release];
    [customCellView     release];

    [cachedSectionViews release];

    [super dealloc];

    END
}

// ========================================================================= //
                #pragma mark table view source/delegate methods
// ========================================================================= //

// =============================== Sections ================================ //

- (NSInteger) numberOfSectionsInTableView: (UITableView*) table_view;
{
    BEGIN
    // WTLOG(@"-tableView: numberOfSectionsInTableView:");
    RETURN NUM_SECTIONS;
    END
}

// ========================================================================= //

- (NSInteger) tableView: (UITableView*) table_view
  numberOfRowsInSection: (NSInteger) section;
{
    BEGIN
    // WTLOG(@"-tableView: numberOfRowsInSection: %i", section);

    switch (section)
    {
        default:
        case kSecSimpleActivity:
            RETURN NUM_SEC_SIMPLE_ACTIVITY_ROWS;

        case kSecBezelActivity:
            RETURN NUM_SEC_BEZEL_ACTIVITY_ROWS;

        case kSecBezelProgress:
            RETURN NUM_SEC_BEZEL_PROGRESS_ROWS;

        case kSecBezelFlexible:
            RETURN NUM_SEC_BEZEL_FLEXIBLE_ROWS;

        case kSecKeyboardActivity:
            RETURN NUM_SEC_KEYBOARD_ACTIVITY_ROWS;

        case kSecKeyboardProgress:
            RETURN NUM_SEC_KEYBOARD_PROGRESS_ROWS;

        case kSecKeyboardFlexible:
            RETURN NUM_SEC_KEYBOARD_FLEXIBLE_ROWS;

        case kSecFunkyFeedbackView:
            RETURN NUM_SEC_FUNKY_FEEDBACK_VIEW_ROWS;
    }

    END
}

// =============================== Headers ================================= //

- (CGFloat) tableView: (UITableView*) table_view
heightForHeaderInSection: (NSInteger) section;
{
    BEGIN
    // WTLOG(@"-tableView: heightForHeaderInSection: %i", section);
    RETURN kRooViewSectionH;
    END
}

// ========================================================================= //

- (UIView*) tableView: (UITableView*) table_view
viewForHeaderInSection: (NSInteger) section;
{
    BEGIN
    // WTLOG(@"-tableView: viewForHeaderInSection: %i", section);

    RootViewSection* sectionView = (RootViewSection*)
        [cachedSectionViews objectAtIndex: section];

    if ((id) sectionView == (id) [NSNull null])
    {
        [[NSBundle mainBundle] loadNibNamed: @"RootViewSection"
                   owner: self options: nil];

        sectionView = customSectionView;
        customSectionView = nil;

        [cachedSectionViews replaceObjectAtIndex: section
                                      withObject: sectionView];

        switch (section)
        {
            default:
            case kSecSimpleActivity:
                sectionView.styleLabel.text = @"Simple";
                sectionView.kindLabel.text  = @"Activity";
                break;

            case kSecBezelActivity:
                sectionView.styleLabel.text = @"Bezel";
                sectionView.kindLabel.text  = @"Activity";
                break;

            case kSecBezelProgress:
                sectionView.styleLabel.text = @"Bezel";
                sectionView.kindLabel.text  = @"Progress";
                break;

            case kSecBezelFlexible:
                sectionView.styleLabel.text = @"Bezel";
                sectionView.kindLabel.text  = @"Flexible";
                break;

            case kSecKeyboardActivity:
                sectionView.styleLabel.text = @"Keyboard";
                sectionView.kindLabel.text  = @"Activity";
                break;

            case kSecKeyboardProgress:
                sectionView.styleLabel.text = @"Keyboard";
                sectionView.kindLabel.text  = @"Progress";
                break;

            case kSecKeyboardFlexible:
                sectionView.styleLabel.text = @"Keyboard";
                sectionView.kindLabel.text  = @"Flexible";
                break;

            case kSecFunkyFeedbackView:
                sectionView.styleLabel.text = @"Funky";
                sectionView.kindLabel.text  = @"Flexible";
                break;
        }
    }

    RETURN sectionView;
    END
}

// ================================= Rows ================================== //

- (UITableViewCell*) tableView: (UITableView*) table_view
cellForRowAtIndexPath: (NSIndexPath*) index_path;
{
    BEGIN
    // WTLOG(@"-tableView: cellForRowAtIndexPath: (%i, %i)", index_path.section, index_path.row);

    NSInteger section = index_path.section;
    NSInteger row     = index_path.row;

    RootViewCell* cell = [self tableView: table_view
                      loadCellForSection: section row: row];

     cell.contentLabel.text = @"";

     cell.method1Label.text = @"";
     cell.method2Label.text = @"";
     cell.method3Label.text = @"";

    switch (section)
    {
        default:
        case kSecSimpleActivity:
            switch (row)
            {
                default:
                case kSecSimpleActivityVFV:
                    cell.contentLabel.text = @"with the default text";
                    cell.method1Label.text = @"+newSimpleActivityViewForView:";
                break;

                case kSecSimpleActivityVFV_WT:
                    cell.contentLabel.text = @"with customized text";
                    cell.method1Label.text = @"+newSimpleActivityViewForView:";
                    cell.method2Label.text = @"withText:";
                break;
            }
            break;

        case kSecBezelActivity:
            switch (row)
            {
                default:
                case kSecBezelActivityVFV:
                    cell.contentLabel.text = @"with the default text";
                    cell.method1Label.text = @"+newBezelActivityViewForView:";
                break;

                case kSecBezelActivityVFV_WT:
                    cell.contentLabel.text = @"with customized text";
                    cell.method1Label.text = @"+newBezelActivityViewForView:";
                    cell.method2Label.text = @"withText:";
                break;
            }
            break;

        case kSecBezelProgress:
            switch (row)
            {
                default:
                case kSecBezelProgressVFV:
                    cell.contentLabel.text = @"with the default text";
                    cell.method1Label.text = @"+newBezelProgressViewForView:";
                break;

                case kSecBezelProgressVFV_WT:
                    cell.contentLabel.text = @"with customized text";
                    cell.method1Label.text = @"+newBezelProgressViewForView:";
                    cell.method2Label.text = @"withText:";
                break;
            }
            break;

        case kSecBezelFlexible:
            switch (row)
            {
                default:
                case kSecBezelFlexibleVFV:
                    cell.contentLabel.text = @"with the default text";
                    cell.method1Label.text = @"+newBezelFlexibleViewForView:";
                    cell.method2Label.text = @"showingProgressView:";
                break;

                case kSecBezelFlexibleVFV_WT:
                    cell.contentLabel.text = @"with customized text";
                    cell.method1Label.text = @"+newBezelFlexibleViewForView:";
                    cell.method2Label.text = @"showingProgressView:";
                    cell.method3Label.text = @"withText:";
                break;
            }
            break;

        case kSecKeyboardActivity:
            switch (row)
            {
                default:
                case kSecKeyboardActivityVFV:
                    cell.contentLabel.text = @"with the default text";
                    cell.method1Label.text = @"+newKeyboardActivityView";
                break;

                case kSecKeyboardActivityVFV_WT:
                    cell.contentLabel.text = @"with customized text";
                    cell.method1Label.text =
                        @"+newKeyboardActivityViewWithText:";
                break;
            }
            break;

        case kSecKeyboardProgress:
            switch (row)
            {
                default:
                case kSecKeyboardProgressVFV:
                    cell.contentLabel.text = @"with the default text";
                    cell.method1Label.text = @"+newKeyboardProgressView";
                break;

                case kSecKeyboardProgressVFV_WT:
                    cell.contentLabel.text = @"with customized text";
                    cell.method1Label.text =
                        @"+newKeyboardProgressViewWithText:";
                break;
            }
            break;

        case kSecKeyboardFlexible:
            switch (row)
            {
                default:
                case kSecKeyboardFlexibleVFV:
                    cell.contentLabel.text = @"with the default text";
                    cell.method1Label.text =
                        @"+newKeyboardFlexibleViewShowingProgressView:";
                break;

                case kSecKeyboardFlexibleVFV_WT:
                    cell.contentLabel.text = @"with customized text";
                    cell.method1Label.text =
                        @"+newKeyboardFlexibleViewShowingProgressView:";
                    cell.method2Label.text = @"withText:";
                break;
            }
            break;

        case kSecFunkyFeedbackView:
            switch (row)
            {
                default:
                case kSecFunkyFeedbackViewVFV_WT:
                    cell.contentLabel.text = @"an example of subclassing WTFeedbackView";
                    cell.method1Label.text = @"+newFunkyViewForView:";
                    cell.method2Label.text = @"showingProgressView:";
                    cell.method3Label.text = @"withText:";
                break;
            }
            break;
    }

    RETURN cell;
    END
}

// ========================================================================= //

- (CGFloat) tableView: (UITableView*) table_view
heightForRowAtIndexPath: (NSIndexPath*) index_path;
{
    BEGIN
    // WTLOG(@"-tableView: heightForRowAtIndexPath: (%i, %i)", index_path.section, index_path.row);

    NSInteger section = index_path.section;
    NSInteger row     = index_path.row;

    switch (section)
    {
        default:
        case kSecSimpleActivity:
            switch (row)
            {
                default:
                case kSecSimpleActivityVFV:
                    RETURN kRootViewCell1H;

                case kSecSimpleActivityVFV_WT:
                    RETURN kRootViewCell2H;
            }
            break;

        case kSecBezelActivity:
            switch (row)
            {
                default:
                case kSecBezelActivityVFV:
                    RETURN kRootViewCell1H;

                case kSecBezelActivityVFV_WT:
                    RETURN kRootViewCell2H;
            }
            break;

        case kSecBezelProgress:
            switch (row)
            {
                default:
                case kSecBezelProgressVFV:
                    RETURN kRootViewCell1H;

                case kSecBezelProgressVFV_WT:
                    RETURN kRootViewCell2H;
            }
            break;

        case kSecBezelFlexible:
            switch (row)
            {
                default:
                case kSecBezelFlexibleVFV:
                    RETURN kRootViewCell2H;

                case kSecBezelFlexibleVFV_WT:
                    RETURN kRootViewCell3H;
            }
            break;

        case kSecKeyboardActivity:
            switch (row)
            {
                default:
                case kSecKeyboardActivityVFV:
                    RETURN kRootViewCell1H;

                case kSecKeyboardActivityVFV_WT:
                    RETURN kRootViewCell1H;
            }
            break;

        case kSecKeyboardProgress:
            switch (row)
            {
                default:
                case kSecKeyboardProgressVFV:
                    RETURN kRootViewCell1H;

                case kSecKeyboardProgressVFV_WT:
                    RETURN kRootViewCell1H;
            }
            break;

        case kSecKeyboardFlexible:
            switch (row)
            {
                default:
                case kSecKeyboardFlexibleVFV:
                    RETURN kRootViewCell1H;

                case kSecKeyboardFlexibleVFV_WT:
                    RETURN kRootViewCell2H;
            }
            break;

        case kSecFunkyFeedbackView:
            switch (row)
            {
                default:
                case kSecFunkyFeedbackViewVFV_WT:
                    RETURN kRootViewCell3H;
            }
            break;
    }

    END
}

// =============================== Selecting =============================== //

- (void) tableView: (UITableView*) table_view
didSelectRowAtIndexPath: (NSIndexPath*) index_path;
{
    BEGIN
    // WTLOG(@"-tableView: didSelectRowAtIndexPath: (%i, %i)", index_path.section, index_path.row);

    DemoViewController* demoVC = [[DemoViewController alloc]
                initWithNibName: @"DemoViewController"
                         bundle: [NSBundle mainBundle]];

    RootViewCell* cell = (RootViewCell*)
        [table_view cellForRowAtIndexPath: index_path];

    NSInteger section = index_path.section;
    NSInteger row     = index_path.row;

    demoVC.section = section;
    demoVC.row     = row;

    demoVC.contentText = cell.contentLabel.text;

    demoVC.method1Text = cell.method1Label.text;
    demoVC.method2Text = cell.method2Label.text;
    demoVC.method3Text = cell.method3Label.text;

     RootViewSection* sectionView = (RootViewSection*)
        [cachedSectionViews objectAtIndex: section];

    demoVC.styleText = sectionView.styleLabel.text;
    demoVC.kindText  = sectionView.kindLabel.text;

    // The HIGs recommend that rows do not stay selected.
    [table_view deselectRowAtIndexPath: index_path animated: YES];

    [self.navigationController pushViewController: demoVC animated: YES];
	[demoVC release];

    END
}

// ========================================================================= //
                        #pragma mark private methods
// ========================================================================= //

- (RootViewCell*) tableView: (UITableView*) table_view
         loadCellForSection: (NSInteger) section
                        row: (NSInteger) row;
{
    BEGIN
    // WTLOG(@"-tableView: loadCellForSection: row: (%i, %i)", section, row);

    switch (section)
    {
        default:
        case kSecSimpleActivity:
            switch (row)
            {
                default:
                case kSecSimpleActivityVFV:
                    RETURN [self tableView: table_view
                                loadCellID: @"1"
                                  nibNamed: @"RootViewCell1"];

                case kSecSimpleActivityVFV_WT:
                    RETURN [self tableView: table_view
                                loadCellID: @"2"
                                  nibNamed: @"RootViewCell2"];
            }
            break;

        case kSecBezelActivity:
            switch (row)
            {
                default:
                case kSecBezelActivityVFV:
                    RETURN [self tableView: table_view
                                loadCellID: @"1"
                                  nibNamed: @"RootViewCell1"];

                case kSecBezelActivityVFV_WT:
                    RETURN [self tableView: table_view
                                loadCellID: @"2"
                                  nibNamed: @"RootViewCell2"];
            }
            break;

        case kSecBezelProgress:
            switch (row)
            {
                default:
                case kSecBezelProgressVFV:
                    RETURN [self tableView: table_view
                                loadCellID: @"1"
                                  nibNamed: @"RootViewCell1"];

                case kSecBezelProgressVFV_WT:
                    RETURN [self tableView: table_view
                                loadCellID: @"2"
                                  nibNamed: @"RootViewCell2"];
            }
            break;

        case kSecBezelFlexible:
            switch (row)
            {
                default:
                case kSecBezelFlexibleVFV:
                    RETURN [self tableView: table_view
                                loadCellID: @"2"
                                  nibNamed: @"RootViewCell2"];

                case kSecBezelFlexibleVFV_WT:
                    RETURN [self tableView: table_view
                                loadCellID: @"3"
                                  nibNamed: @"RootViewCell3"];
            }
            break;

        case kSecKeyboardActivity:
            switch (row)
            {
                default:
                case kSecKeyboardActivityVFV:
                    RETURN [self tableView: table_view
                                loadCellID: @"1"
                                  nibNamed: @"RootViewCell1"];

                case kSecKeyboardActivityVFV_WT:
                    RETURN [self tableView: table_view
                                loadCellID: @"1"
                                  nibNamed: @"RootViewCell1"];
            }
            break;

        case kSecKeyboardProgress:
            switch (row)
            {
                default:
                case kSecKeyboardProgressVFV:
                    RETURN [self tableView: table_view
                                loadCellID: @"1"
                                  nibNamed: @"RootViewCell1"];

                case kSecKeyboardProgressVFV_WT:
                    RETURN [self tableView: table_view
                                loadCellID: @"1"
                                  nibNamed: @"RootViewCell1"];
            }
            break;

        case kSecKeyboardFlexible:
            switch (row)
            {
                default:
                case kSecKeyboardFlexibleVFV:
                    RETURN [self tableView: table_view
                                loadCellID: @"1"
                                  nibNamed: @"RootViewCell1"];

                case kSecKeyboardFlexibleVFV_WT:
                    RETURN [self tableView: table_view
                                loadCellID: @"2"
                                  nibNamed: @"RootViewCell2"];
            }
            break;

        case kSecFunkyFeedbackView:
            switch (row)
            {
                default:
                case kSecFunkyFeedbackViewVFV_WT:
                    RETURN [self tableView: table_view
                                loadCellID: @"3"
                                  nibNamed: @"RootViewCell3"];
            }
            break;
    }

    END
}

// ========================================================================= //

- (RootViewCell*) tableView: (UITableView*) table_view
                 loadCellID: (NSString*) cellID
                   nibNamed: (NSString*) nibName;
{
    BEGIN
    // WTLOG(@"-tableView: loadCellID: %@ nibNamed: %@", cellID, nibName);

    RootViewCell* cell = (RootViewCell*) [table_view
        dequeueReusableCellWithIdentifier: cellID];

    if (cell == nil)
    {
        [[NSBundle mainBundle] loadNibNamed: nibName owner: self options: nil];

        cell = [[customCellView retain] autorelease];
        self.customCellView = nil;
    }

    RETURN cell;
    END
}

// ========================================================================= //

@end
